clc;
clear all;
close;
%******************************************************************************************************************
%----calculate the viscosity based on the method of Shiyaolin's paper-----------------------------------------------
%----------------------by XUJing in July 14, 2018-----------------------------------------------------------------------
% data requirement: temperature strain rate parameters of A,n,E crust structure(crustal thicknesscrust1.0)
%******************************************************************************************************************
%% --------input parameters which need to be set------------------
%---------the location of the study area,the range of the result-----------
lonmax=106;                                                                % study area
lonmin=96;
latmax=33;
latmin=20;
%---------input data-------------------------------------------------------
tem=load('temperature_sun2013_input.txt');                                 % temperature data: No. lon lat depth(m) temperature(C)
strainrate=load('strainrate_wang2017_input.dat');                          % strainrate data: lat lon strainrate[10E-8*yr(-1)] direction
crustalthickness=load('crustalthickness_crust1.0.xyz');                    % lon lat thickness
temNo=1;   temlon=2;   temlat=3;   temdep=4;   temtem=5;                   % the meaning of each column of input files
strlon=2;  strlat=1;   strstr=3;   strdir=4;
ctlon=1;   ctlat=2;    ctthick=3;
%---------the creep parameters (Shi and Cao,2008)--------------------------% AMPa^(-n*s^(-1), E:KJ *mol^(-1)
% A1=1.995*10^(-4);   n1=1.9;    E1=141;                                     % upper crust
% A2=3.8*10^(-2);     n2=2.4;    E2=219;                                     % middle crust
% A3=8*10^(-3);       n3=3.1;    E3=243;                                     % upper lower crust
% A4=1.4*10^4;        n4=4.2;    E4=445;                                     % lowermost crust
% A5=2.0*10^3;        n5=4.0;    E5=471;                                     % upper mantle
% R=8.314;                                                                   % gas constant J/mol/K    J*mol^(-1)*k^(-1) 
%---------the creep parameters (Sun et al., 2013)--------------------------% AMPa^(-n*s^(-1), E:KJ *mol^(-1)
A1=1.8*10^(-9);     n1=3.2;    E1=123;                                     % upper crust
A2=3.2*10^(-4);     n2=3.2;    E2=238;                                     % middle crust
A3=8*10^(-3);       n3=3.1;    E3=243;                                     % upper lower crust
A4=1.4*10^4;        n4=4.2;    E4=445;                                     % lowermost crust
A5=2.0*10^3;        n5=4.0;    E5=471;                                     % upper mantle
R=8.314;                                                                   % gas constant J/mol/K    J*mol^(-1)*k^(-1) 
%% ------output parameters---------------------------------------------------
file=3;                                                                    % 1:depth  2:profile  3:point
%------all point (3D result in study area) lat lon depth-------------------
outputdata=[];                                                             % lat,lon,depth,temperature,strainrate,thickness,strength,viscosity,log10(viscosity)
%------2D result in specific depth (lat lon)-------------------------------
outputdata_depth=[];                                                       % lat,lon,depth,temperature,strainrate,thickness,strength,viscosity,log10(viscosity)
depth=80;                                                                 % get the results in specific depth
%------profile  (distance depth)-------------------------------------------
lonStart=96.5;                                                             % start and end point of the profile
latStart=24.5;
lonEnd=105;
latEnd=30.5;
depthup=0;
depthdown=100;
lineNum=201;                                                               % number of points per line 
depthNum=101;                                                              % number of depth
outputdata_profile=[];                                                     % lat,lon,depth,temperature,strainrate,thickness,strength,viscosity,log10(viscosity) distance
%---------one point--------------------------------------------------------
lonP=105;
latP=30;
depdown=100;                                                               % bottom
depNum=10000;                                                              % depth points
outputdata_point=[];                                                       % lat,lon,depth,temperature,strainrate,thickness,strength,viscosity,log10(viscosity)
%% ------data preprocessing for all point-------------------------
%------filter the temperature and the strainrate inputdata in the study area----------
lentem=size(tem,1);
lenstr=size(strainrate,1);
tem_study=[];                                                              % store the temperature data in study area
str_study=[];                                                              % store the strain rate data in study area
count_tem=1;
count_str=1;
for i=1:lentem                                                             % get the temperature point in study area 
    if tem(i,temlon)>=lonmin&&tem(i,temlon)<=lonmax&&tem(i,temlat)>=latmin&&tem(i,temlat)<=latmax
        tem_study(count_tem,:)=tem(i,:);
        count_tem=count_tem+1;
    end
end
for j=1:lenstr                                                             % get the strain rate point in study area
    if strainrate(j,strlon)>=lonmin&&strainrate(j,strlon)<=lonmax&&strainrate(j,strlat)>=latmin&&strainrate(j,strlat)<=latmax
        str_study(count_str,:)=strainrate(j,:);
        count_str=count_str+1;
    end
end
%------get the 3D coordinate,temperature,strainrate,thickness of the all point in the study area using the same grid with the input temperature file-------
loc_1layer=[];                                                             % get the location (lat lon) of the 1layer to grid
count_dep=1;                                                               % same 2D locations for every layer,but the depth is diferent in one layer
for k=1:count_tem-1
    if tem_study(1,temdep)==tem_study(k,temdep)
       loc_1layer(count_dep,1)=tem_study(k,temlat);
       loc_1layer(count_dep,2)=tem_study(k,temlon);
       count_dep=count_dep+1;
    end
end
[loc_1layer(:,1),loc_1layer(:,2),str]=griddata(str_study(:,strlat),str_study(:,strlon),str_study(:,strstr),loc_1layer(:,1),loc_1layer(:,2),'nearest');  
str_1layer=[loc_1layer(:,1),loc_1layer(:,2),str];                          % get the grid of the strain rate in 1layer

for m=1:count_tem-1                                                        % outputdata: lat,lon,depth,temperature,strainrate
           outputdata(m,1)=tem_study(m,temlat);
           outputdata(m,2)=tem_study(m,temlon);
           outputdata(m,3)=tem_study(m,temdep)/1000;                       % m to km
           outputdata(m,4)=tem_study(m,temtem)+273.15;                     % C to K absolute temperature
    for n=1:size(str_1layer,1)                                             % write the strainrate in each point to the outputdata
        if tem_study(m,temlat)==str_1layer(n,1)&&tem_study(m,temlon)==str_1layer(n,2)
           outputdata(m,5)=str_1layer(n,3)*(1.0E-15)/3.1536;               % yr(-1) to s(-1)
        end
    end
end
    %----------grid the crustal thickness in each cal point----------------
     outputdata(:,6)=griddata(crustalthickness(:,ctlat),crustalthickness(:,ctlon),crustalthickness(:,ctthick),outputdata(:,1),outputdata(:,2),'nearest');   % outputdata: lat,lon,depth,temperature,strainrate thickness
     
%% ------data preprocessing for 2D results in specific depth-------------------------
if file==1
outputdata_depth(:,1)=loc_1layer(:,1);                                     % write lat in outputdata_depth
outputdata_depth(:,2)=loc_1layer(:,2);                                     % write lon in outputdata_depth
for t=1:size(outputdata_depth,1)
    outputdata_depth(t,3)=-depth;                                          % specific depth for outputdata_depth
end
outputdata_depth(:,4)=griddata(outputdata(:,1),outputdata(:,2),outputdata(:,3),outputdata(:,4),outputdata_depth(:,1),outputdata_depth(:,2),outputdata_depth(:,3)); % temperature
outputdata_depth(:,5)=griddata(str_study(:,strlat),str_study(:,strlon),str_study(:,strstr),outputdata_depth(:,1),outputdata_depth(:,2),'nearest');                 % strain rate
outputdata_depth(:,5)=outputdata_depth(:,5)*(1.0E-15)/3.1536;              % yr(-1) to s(-1)
outputdata_depth(:,6)=griddata(crustalthickness(:,2),crustalthickness(:,1),crustalthickness(:,3),outputdata_depth(:,1),outputdata_depth(:,2));                     % thickness
end
%% ------data preprocessing for profile---------------------------
if file==2
linelat=linspace(latStart,latEnd,lineNum);
linelon=linspace(lonStart,lonEnd,lineNum);
linedepth=linspace(depthup,depthdown,depthNum);
for u=1:depthNum
    outputdata_profile((u-1)*lineNum+1:u*lineNum,1)=linelat';
    outputdata_profile((u-1)*lineNum+1:u*lineNum,2)=linelon';
    outputdata_profile((u-1)*lineNum+1:u*lineNum,3)=-linedepth(u);
end
outputdata_profile(:,4)=griddata(outputdata(:,1),outputdata(:,2),outputdata(:,3),outputdata(:,4),outputdata_profile(:,1),outputdata_profile(:,2),outputdata_profile(:,3)); % temperature
outputdata_profile(:,5)=griddata(str_study(:,strlat),str_study(:,strlon),str_study(:,strstr),outputdata_profile(:,1),outputdata_profile(:,2),'nearest');                   % strain rate
outputdata_profile(:,5)=outputdata_profile(:,5)*(1.0E-15)/3.1536;              % yr(-1) to s(-1)
outputdata_profile(:,6)=griddata(crustalthickness(:,2),crustalthickness(:,1),crustalthickness(:,3),outputdata_profile(:,1),outputdata_profile(:,2));                       % thickness
end
%% -------data preprocessing for one point------------------------
if file==3
surfP=griddata(outputdata(count_tem-size(loc_1layer,1):count_tem-1,1),outputdata(count_tem-size(loc_1layer,1):count_tem-1,2),outputdata(count_tem-size(loc_1layer,1):count_tem-1,3),latP,lonP); % surface elevation in the point
linedep=linspace(-surfP,depdown,depNum);                                   % cal_point in depth
for pp=1:size(linedep',1)                                                  % prepare the cal_point coordinate (lat lon depth)
    outputdata_point(pp,1)=latP;
    outputdata_point(pp,2)=lonP;
    outputdata_point(pp,3)=-linedep(pp);
end
outputdata_point(:,4)=griddata(outputdata(:,1),outputdata(:,2),outputdata(:,3),outputdata(:,4),outputdata_point(:,1),outputdata_point(:,2),outputdata_point(:,3)); % temperature
outputdata_point(:,5)=griddata(str_study(:,strlat),str_study(:,strlon),str_study(:,strstr),outputdata_point(:,1),outputdata_point(:,2),'nearest');                 % strain rate
outputdata_point(:,5)=outputdata_point(:,5)*(1.0E-15)/3.1536;              % yr(-1) to s(-1)
outputdata_point(:,6)=griddata(crustalthickness(:,2),crustalthickness(:,1),crustalthickness(:,3),outputdata_point(:,1),outputdata_point(:,2));                     % thickness in the point                   % thickness
end
%% ------calculate the strength-----------------------------------

%---*******------------*******---
    if file==1
        data=outputdata_depth;
    elseif file==2
        data=outputdata_profile;
    elseif file==3
        data=outputdata_point;
    end
%---*******------------*******---

    strength=[];
    %--------1st method to calculate the strength------------------------------
    for p=1:size(data,1)
        surf1=griddata(outputdata(count_tem-size(loc_1layer,1):count_tem-1,1),outputdata(count_tem-size(loc_1layer,1):count_tem-1,2),outputdata(count_tem-size(loc_1layer,1):count_tem-1,3),data(p,1),data(p,2)); % surface elevation in the point
        if (-data(p,3)+surf1)<12.8                                           % depth km
            strength(p,1)=(34.91*(-data(p,3)+surf1)/2)*1.0E+6;               % Pa ǿȿΪֵ𣿣رиǲĻ
        elseif (-data(p,3)+surf1)>=12.8&&(-data(p,3)+surf1)<102.0
                strength(p,1)=((27.73*(-data(p,3)+surf1)+85.75)/2)*1.0E+6;   % Pa
        else strength(p,1)=1.0E+30;                                                % always greater than the result of the method 2
        end
    end
    %--------2nd method to calculate the strength------------------------------
    for q=1:size(data,1)
       %----------find the surface depth--------------------------------------
        surf=griddata(outputdata(count_tem-size(loc_1layer,1):count_tem-1,1),outputdata(count_tem-size(loc_1layer,1):count_tem-1,2),outputdata(count_tem-size(loc_1layer,1):count_tem-1,3),data(q,1),data(q,2)); % surface elevation in the point
        %---------creep parameters determined by the crust structure----0.35/0.25/0.25/0.15-------
        L1up=surf;                                L1low=-(data(q,6)*0.35-surf);
        L2up=-(data(q,6)*0.35-surf);              L2low=-(data(q,6)*(0.35+0.25)-surf);
        L3up=-(data(q,6)*(0.35+0.25)-surf);       L3low=-(data(q,6)*(0.35+0.25+0.25)-surf);
        L4up=-(data(q,6)*(0.35+0.25+0.25)-surf);  L4low=-(data(q,6)*(0.35+0.25+0.25+0.15)-surf);
        L5up=-(data(q,6)-surf);
        if data(q,3)>L1low&&data(q,3)<=L1up
            A=A1;  n=n1;  E=E1;
        elseif data(q,3)>L2low&&data(q,3)<=L2up
            A=A2;  n=n2;  E=E2;
        elseif data(q,3)>L3low&&data(q,3)<=L3up
            A=A3;  n=n3;  E=E3;
        elseif data(q,3)>L4low&&data(q,3)<=L4up
            A=A4;  n=n4;  E=E4;
        elseif data(q,3)<=L5up
            A=A5;  n=n5;  E=E5;
        end
        %-------strength-------------------------------------------------------
        strength(q,2)=(((data(q,5)/A)^(1/n))*exp(E/(n*(R*1.0E-3)*data(q,4)))/2)*1.0E+6;    % Pa
    end

    for s=1:size(data,1)
        data(s,7)=min(strength(s,:));
        data(s,8)=data(s,7)/data(s,5);
        data(s,9)=log10(data(s,8));
    end
%---*******------------*******---
    if file==1
        outputdata_depth=data;
    elseif file==2
        outputdata_profile=data;
        for w=1:size(outputdata_profile,1)
            y=(outputdata_profile(w,1)-latStart)*111.199;
            x=(outputdata_profile(w,2)-lonStart)*(111.199*cos(((latStart+outputdata_profile(w,1))/2)*pi/180));
            outputdata_profile(w,10)=sqrt((x*x)+(y*y));
        end

    elseif file==3
        outputdata_point=data;
    end    
%---*******------------*******---

%% ----------output-----------------------------------------------
% dlmwrite('output.txt',outputdata,'delimiter',' ');
if file==1
    dlmwrite(sprintf('%s%s%s','output_',num2str(depth),'.txt'),outputdata_depth,'delimiter',' ');
elseif file==2
    dlmwrite('output_profile.txt',outputdata_profile,'delimiter',' ');
elseif file==3
    plot(outputdata_point(:,9),outputdata_point(:,3));
    axis([19 25 -100 5]);
end